@echo off
@setlocal enabledelayedexpansion

set bin=%~dp0
set grafana_home=%~1
set plugin_name=da2-app

whoami /priv | find "SeLoadDriverPrivilege" > nul 2>&1
if %errorlevel% neq 0 (
    call :logger "execute setup.bat by administrator"
    exit /b 1
)

if "%grafana_home%"=="" (
    set /p input="Input Grafana install path: "
    call :update_grafana_home !input!
    if "!grafana_home!"=="" (
        call :logger "Grafana install path is not set" ERROR
        exit /b 1
    )
)
rem if end of grafana_home is '\', remove '\'
if "%grafana_home:~-1%"=="\" (
    set grafana_home=%grafana_home:~0,-1%
)

rem if end of bin is '\', remove '\'
if "%bin:~-1%"=="\" (
    set bin=%bin:~0,-1%
)

if not exist "%grafana_home%" (
    call :logger "%grafana_home% is not exist" ERROR
    exit /b 1
)

if not exist "%bin%\datasource.yml" (
    call :logger "%bin%\datasource.yml is not exist" ERROR
    exit /b 1
)

set plugin_dir=%grafana_home%\data\plugins
set provisioning_dir=%grafana_home%\conf\provisioning

if exist "%plugin_dir%\%plugin_name%" (
    call :logger "plugin is already installed." ERROR
    call :logger "delete %plugin_dir%\%plugin_name% if you want to reinstall plugin." ERROR
    exit /b 1
)

if not exist "%grafana_home%\conf\sample.ini" (
    call :logger "%grafana_home%\conf\sample.ini is not exist" ERROR
    exit /b 1
)

if exist "%grafana_home%\conf\custom.ini" (
    copy /y "%grafana_home%\conf\custom.ini" "%grafana_home%\conf\custom.ini.back"

    if not !errorlevel!==0 (
        call :logger "Cannot backup Grafana setting file" ERROR
        exit /b 1
    )
)

copy /y "%grafana_home%\conf\sample.ini" "%grafana_home%\conf\custom.ini"
if not %errorlevel%==0 (
    call :error_handling "Cannot create %grafana_home%\conf\custom.ini"
    exit /b 1
)

type nul > "%grafana_home%\conf\custom.ini"
if not %errorlevel%==0 (
    call :error_handling "Cannot create %grafana_home%\conf\custom.ini"
    exit /b 1
)

rem modify Grafana setting file
rem add row number for each line (e.g. "1:;http_addr = 0.0.0.1") and split to
rem %%a ...row number (e.g. "1")
rem %%b ...right side (e.g. ";http_addr")
rem %%c ...left side (e.g. "= 0.0.0.1")
for /f "tokens=1,2* eol= delims=: " %%a in ( 'findstr /n "^" "%grafana_home%\conf\sample.ini"') do (
    if "%%b"==";http_addr" (
        (echo http_addr = 127.0.0.1) >> "%grafana_home%\conf\custom.ini"
    ) else if "%%b"==";allow_loading_unsigned_plugins" (
        (echo allow_loading_unsigned_plugins = da2-app,da2-app-datasource,da2-app-trackmap-panel) >> "%grafana_home%\conf\custom.ini"
    ) else (
        if "%%c"=="" (
            (echo.%%b) >> "%grafana_home%\conf\custom.ini"
        ) else (
            (echo %%b %%c) >> "%grafana_home%\conf\custom.ini"
        )
    )
)

rem install plugin
if not exist "%plugin_dir%" (
    mkdir "%plugin_dir%"
    if not !errorlevel!==0 (
        call :error_handling "Cannot create %plugin_dir%"
        exit /b 1
    )
) 
pushd "%bin%"
start /wait %plugin_name%.exe -y
if not %errorlevel%==0 (
    call :error_handling "Cannot extract %plugin_name%"
    exit /b 1
)
xcopy /s /i %plugin_name% "%plugin_dir%\%plugin_name%"
if not %errorlevel%==0 (
    call :error_handling "Cannot copy %bin%\%plugin_name% to %plugin_dir%"
    rd /s /q %plugin_name%
    exit /b 1
)
rd /s /q %plugin_name%

popd

rem copy datasource.yml
if not exist "%provisioning_dir%\datasources" (
    mkdir "%provisioning_dir%\datasources"
    if not !errorlevel!==0 (
        call :error_handling "Cannot create %provisioning_dir%\datasources"
        exit /b 1
    )
) 
copy "%bin%\datasource.yml" "%provisioning_dir%\datasources"
if not %errorlevel%==0 (
    call :error_handling "Cannot copy %bin%\datasource.yml"
    exit /b 1
)

rem copy plugin.yml
if exist "%provisioning_dir%\plugins" (
    copy "%bin%\plugin.yml" "%provisioning_dir%\plugins"
)

tasklist | find "grafana-server.exe" > nul 2>&1
if %errorlevel%==0 (
    taskkill /im grafana-server.exe
    if not !errorlevel!==0 (
        call :error_handling "Cannot stop Grafana." ERROR
        exit /b 1
    )
)

pushd "%grafana_home%\bin"
start grafana-server.exe
if not %errorlevel%==0 (
    call :error_handling "Cannot start Grafana. See log file %grafana_home%\data\log" ERROR
    exit /b 1
)
call :logger "import plugin and datasource. wait 1 minutes."

timeout 60 > nul
popd

del "%provisioning_dir%\datasources\datasource.yml"

tasklist | find "grafana-server.exe" > nul 2>&1
if not %errorlevel%==0 (
    call :error_handling "Cannot start Grafana. See log file %grafana_home%\data\log" ERROR
    exit /b 1
)

call :logger "setup is done. Grafana is started."

exit /b 0


rem Usage -> call :error_handling "<error message>"
:error_handling
if not "%~1"=="" (
    call :logger %1 ERROR
)

if exist "%grafana_home%\conf\custom.ini.back" (
    del "%grafana_home%\conf\custom.ini"
    call :get_file_size "%grafana_home%\conf\custom.ini.back"
    if not !error_level!==0 (
        copy /y "%grafana_home%\conf\custom.ini.back" "%grafana_home%\conf\custom.ini" > nul
    )
)

if exist "%plugin_dir%\%plugin_name%" (
    rd /s /q "%plugin_dir%\%plugin_name%"
)

if exist "%provisioning_path%\datasources\datasource.yml" (
    del "%provisioning_path%\datasources\datasource.yml"
)

if exist "%provisioning_path%\plugins\plugin.yml" (
    del "%provisioning_path%\plugins\plugin.yml"
)

exit /b


rem Usage -> call :logger "<error message>" <error level>
:logger
if not "%~1"=="" (
    if not "%~2"=="" (
        echo %~2 : %~1
    )else (
        echo INFO: %~1
    )
)

exit /b

rem Usage -> call :get_file_size <file_path>
:get_file_size
set file_size=0
if not "%~1"=="" (
    set file_size=%~z1
)

exit /b %file_size%

:update_grafana_home
if not "%~1"=="" (
    set grafana_home=%~1
)

exit /b
